.section .init
.globl _start 
#_start:
    ldr pc,reset_handler
    ldr pc,undefined_handler
    ldr pc,swi_handler
    ldr pc,prefetch_handler
    ldr pc,data_handler
    ldr pc,unused_handler
    ldr pc,irq_handler
    ldr pc,fiq_handler

reset_handler: 		.word reset
undefined_handler: 	.word undefined
swi_handler: 		.word swi
prefetch_handler: 	.word prefetch
data_handler: 		.word data
unused_handler: 	.word unused
irq_handler: 		.word irq
fiq_handler: 		.word fiq
	
_start:
#Copy the interrupts vector to the first memory positions
reset:
	#The stack address must be the same that the one used in the linker script
    mov r0,$0x8000
    mov r1,$0x0000
    ldmia r0!,{r2,r3,r4,r5,r6,r7,r8,r9}
    stmia r1!,{r2,r3,r4,r5,r6,r7,r8,r9}
    ldmia r0!,{r2,r3,r4,r5,r6,r7,r8,r9}
    stmia r1!,{r2,r3,r4,r5,r6,r7,r8,r9}

	# Set the stack. 
	#The stack address must be the same that the one used in the linker script
	mov sp, $0x8000

#set stack in irq mode
        mrs r0, cpsr
        bic r0, $0b11111111
        orr r0, $0b11010010
        msr cpsr, r0
        mov sp, $0x300
 
#come back to svc mode
        mrs r0, cpsr
        bic r0, $0b11111111
        orr r0, $0b11010011
        msr cpsr, r0
	
#fill .bss section with zeros
	ldr r0, =_bss_start
	ldr r1, =_bss_end
	mov r2, $0
	
bssloop:
        cmp r0, r1
        strlo r2, [r0], $4
        blo bssloop
	

#Led ON
#	bl ledOn

#Go to the main
	bl wrapper_main

# infinite loop
loop:
	b loop

#Irq interrupt
irq:
	ldr sp, =_irq_stack
	push {r11, r12, lr}
	mov r12, sp
	
	# switch to SVM
#	mrs r11, cpsr
#	orr r11, r11, #0x1F
#	msr cpsr, r11
	mrs r11, cpsr
        bic r11, $0b11111111
        orr r11, $0b11010011
        msr cpsr, r11
	
	#save system state
	push {r0-r12, lr}

	
	#save interrupt state on system sp
	#ldmdb r12, {r0, r1, r2}
	#stmfd sp!, {r0, r1, r2}
	ldmfd r12!, {r0-r2}
	stmfd sp!, {r0-r2}

	#call the irq interrupt handler
	bl irq_interrupt

	#restore interrupt state from system sp
	ldr r12, =_irq_stack
	#ldmfd sp!, {r0, r1, r2}
	#stmdb r12, {r0, r1, r2}
	ldmfd sp!, {r0-r2}
	stmfd r12!, {r0-r2}

	#restore system state
	pop {r0-r12, lr}

	#switch to interrupt mode
#	mrs r11, cpsr
#	orr r11, r11, #0x1F
#	and r11, r11, #0x12
#	msr cpsr, r11
	mrs r11, cpsr
        bic r11, $0b11111111
        orr r11, $0b11010010
        msr cpsr, r11

	#restore interrupt status
	pop {r11, r12, lr}

	#return from interrupt
	subs pc, lr, $4
	
#Unexpected interrupt
#unexpected:
#	push {r0-r12, lr}
#	bl unexpected_interrupt
#	pop {r0-r12, lr}
#	subs pc, lr, $4

undefined:
        push {r0-r12, lr}
        bl undefined_interrupt
        pop {r0-r12, lr}
        subs pc, lr, $4
swi:
        push {r0-r12, lr}
        bl swi_interrupt
        pop {r0-r12, lr}
        subs pc, lr, $4
prefetch:
        push {r0-r12, lr}
        bl prefetch_interrupt
        pop {r0-r12, lr}
        subs pc, lr, $4
data:
        push {r0-r12, lr}
        bl data_interrupt
        pop {r0-r12, lr}
        subs pc, lr, $4
unused:
        push {r0-r12, lr}
        bl unused_interrupt
        pop {r0-r12, lr}
        subs pc, lr, $4
fiq:
        push {r0-r12, lr}
        bl fiq_interrupt
        pop {r0-r12, lr}
        subs pc, lr, $4
